import { _decorator, Component, Prefab, SpriteFrame, Node, Vec3 } from "cc";
import { WeaponPool } from "./WeaponPool";
import { Weapon } from "./Weapon";

const { ccclass, property } = _decorator;

type WeaponConfig = {
    faction:number,//玩家还是NPC标记
    orbitRadius: number;    // 环绕半径
    skin: string;      // 皮肤贴图
    speed: number;          // 环绕速度
};

@ccclass('WeaponManager')
export class WeaponManager extends Component {
    @property({ type: Prefab })
    weaponPrefab: Prefab | null = null;

    private _activeWeapons: Node[] = [];
    private _currentConfig!: WeaponConfig;

    // 初始化带参数的飞刀系统
    public setup(config: WeaponConfig, initialCount: number) {
        this._currentConfig = config;

        for (let i = 0; i < initialCount; i++) {
            this.createNewWeapon();
        }
        this.updateOrbit();
    }

    // 动态创建飞刀（带参数配置）
    private createNewWeapon() {
        const weapon = WeaponPool.instance.getWeapon(this.weaponPrefab!);
        const weaponComp = weapon.getComponent(Weapon)!;

        // 应用参数配置
        weaponComp.setup({
            faction: this._currentConfig.faction,
            skin: this._currentConfig.skin,
            orbitRadius: this._currentConfig.orbitRadius,
            baseSpeed: this._currentConfig.speed
        });

        this.node.getChildByName("wuqi").addChild(weapon);
        this._activeWeapons.push(weapon);
    }

    // 击飞处理（带距离检测）
    public handleKnockback(weaponNode: Node) {
        const index = this._activeWeapons.indexOf(weaponNode);
        if (index === -1) return;

        // 切换为击飞状态
        const weapon = weaponNode.getComponent(Weapon)!;
        weapon.setState('flyaway');

        weapon.schedule(() => {
            // 使用静态方法计算距离（更高效）
            const distance = Vec3.distance(weaponNode.position, this.node.position);

            if (distance > 500) {
                this.recycleWeapon(weaponNode);
            }
        });
    }

    // 回收飞刀到对象池
    private recycleWeapon(weapon: Node) {
        const index = this._activeWeapons.indexOf(weapon);
        if (index !== -1) {
            this._activeWeapons.splice(index, 1);
            WeaponPool.instance.recycleWeapon(this.weaponPrefab!, weapon);
            this.updateOrbit();
        }
    }

    // 更新环绕布局
    private updateOrbit() {
        const angleStep = 360 / this._activeWeapons.length;
        this._activeWeapons.forEach((weapon, index) => {
            const weaponComp = weapon.getComponent(Weapon)!;
            weaponComp.updateOrbit(angleStep * index);
        });
    }
}
