import { _decorator, instantiate, Prefab,Node } from "cc";
import { Weapon } from "./Weapon";

let { ccclass } = _decorator;

@ccclass('WeaponPool')
export class WeaponPool {
    private static _instance: WeaponPool;
    private _poolDict: Map<string, Node[]> = new Map();

    static get instance() {
        if (!this._instance) {
            this._instance = new WeaponPool();
        }
        return this._instance;
    }

    // 获取/创建飞刀
    public getWeapon(prefab: Prefab): Node {
        let key = prefab.name;
        if (!this._poolDict.has(key)) {
            this._poolDict.set(key, []);
        }

        let pool = this._poolDict.get(key)!;
        if (pool.length > 0) {
            return pool.pop()!;
        } else {
            return instantiate(prefab);
        }
    }

    // 回收飞刀
    public recycleWeapon(prefab: Prefab, weapon: Node) {
        let key = prefab.name;
        weapon.removeFromParent();
        weapon.getComponent(Weapon)?.resetState();
        this._poolDict.get(key)?.push(weapon);
    }
}
