import { _decorator, clamp, Component, Node, RigidBody2D, Vec2, Vec3, PhysicsMaterial, UITransform, director, view, CircleCollider2D } from 'cc';
import { Joystick } from './Joystick';
import { WeaponManager } from './WeaponManager';
let { ccclass, property } = _decorator;

@ccclass('pdRole')
export class pdRole extends Component {
    @property
    moveSpeed: number = 50;

    @property(Node)
    joystickNode: Node = null;

    // 地图边界参数化（中心为 (0,0) 时）
    @property(Vec2)
    private _mapSize: Vec2 = new Vec2(3750, 3750); // 地图总尺寸

    private _mapMin: Vec2 = new Vec2(); // 左下角
    private _mapMax: Vec2 = new Vec2(); // 右上角

    private _rigidBody: RigidBody2D;
    private _joyStickTs: Joystick;
    private _currentVelocity: Vec2 = new Vec2(0, 0);

    // 动态获取碰撞体尺寸
    private _halfWidth: number = 0;
    private _halfHeight: number = 0;

    private _weaponMg: WeaponManager;
    onLoad() {
        //this._rigidBody = this.getComponent(RigidBody2D);
        this._joyStickTs = this.joystickNode.getComponent(Joystick);
        this._weaponMg = this.node.getComponent(WeaponManager);
        this._weaponMg.setup({
            faction: 1,
            orbitRadius: 100,
            skin: "baise",
            speed: 1
        }, 6);
        // 计算地图边界（假设地图中心为 (0,0)）
        this._mapMin.set(-this._mapSize.x / 2, -this._mapSize.y / 2);
        this._mapMax.set(this._mapSize.x / 2, this._mapSize.y / 2);


        let uiTransform = this.getComponent(UITransform);
        if (uiTransform) {
            this._halfWidth = uiTransform.width * 0.5;
            this._halfHeight = uiTransform.height * 0.5;
        }

    }

    update(deltaTime: number) {
        let rawDir = this._joyStickTs.direction;

        if (!this._joyStickTs.isActive || rawDir.length() < 0.1) {
            this._currentVelocity.set(Vec2.ZERO);
            // this._rigidBody.linearVelocity = this._currentVelocity;
            return;
        }
        // let normalizedDir = rawDir.clone().normalize(); // 关键修复点
        // let speedScale = clamp(rawDir.length(), 0.2, 1); // 最低20%速度
        // let targetVelocity = normalizedDir.multiplyScalar(5000 * speedScale);
        // // 插值平滑移动
        // this._currentVelocity.lerp(new Vec2(targetVelocity.x, targetVelocity.y), deltaTime * 10);

        // // 计算新坐标
        // let newPos = this.node.position.clone();
        // newPos.x += this._currentVelocity.x * deltaTime;
        // newPos.y += this._currentVelocity.y * deltaTime;

        // // 立即应用边界限制
        // //this.node.position = this._clampPosition(newPos);
        // // console.log("position", this.node.position);
        // this.node.setPosition(this._clampPosition(newPos));
        // console.log("newPos", newPos);

        // 获取完全归一化方向
        let dir = this._joyStickTs.direction.clone().normalize();

        // 计算瞬时速度
        let velocity = dir.multiplyScalar(this.moveSpeed);
        // 高速移动（直接应用无平滑）
        let newPos2 = new Vec3(
            this.node.position.x + velocity.x ,
            this.node.position.y + velocity.y ,
            0
        );

        this.node.position = this._clampPosition(newPos2);
    }
    /** 边界限制计算 */
    private _clampPosition(pos: Vec3): Vec3 {
        let validMinX = this._mapMin.x + this._halfWidth;
        let validMaxX = this._mapMax.x - this._halfWidth;
        let validMinY = this._mapMin.y + this._halfHeight;
        let validMaxY = this._mapMax.y - this._halfHeight;

        return new Vec3(
            clamp(pos.x, validMinX, validMaxX),
            clamp(pos.y, validMinY, validMaxY),
            pos.z
        );
    }
    lateUpdate() {
        // let pos = this.node.position;

        // // 计算有效移动范围
        // let validMinX = this._mapMin.x + this._halfWidth;
        // let validMaxX = this._mapMax.x - this._halfWidth;
        // let validMinY = this._mapMin.y + this._halfHeight;
        // let validMaxY = this._mapMax.y - this._halfHeight;

        // // 精准边界限制
        // let clampedPos = new Vec3(
        //     clamp(pos.x, validMinX, validMaxX),
        //     clamp(pos.y, validMinY, validMaxY),
        //     pos.z
        // );

        // // 判断是否需要修正位置
        // if (!this._isPositionValid(pos, clampedPos)) {
        //     this.node.setPosition(clampedPos);
        //     this._applyVelocityCorrection(pos, clampedPos);
        // }
    }

    /** 判断位置是否超出容差范围 (1px) */
    private _isPositionValid(current: Vec3, clamped: Vec3): boolean {
        return Math.abs(current.x - clamped.x) <= 1 &&
            Math.abs(current.y - clamped.y) <= 1;
    }

    /** 智能速度修正：仅阻止越界方向的速度 */
    private _applyVelocityCorrection(current: Vec3, clamped: Vec3) {
        let velocity = this._rigidBody.linearVelocity;
        let newVelocity = new Vec2(
            (current.x === clamped.x) ? velocity.x : 0,
            (current.y === clamped.y) ? velocity.y : 0
        );
        this._rigidBody.linearVelocity = newVelocity;
    }
}
